#include <iostream>
#include<pybind11/pybind11.h>        // pybind11的头文件
#include<pybind11/numpy.h>
#include <pybind11/stl.h>

using namespace std;
namespace py = pybind11;               // 名字空间别名，简化代码

typedef int NodeIDType;
// typedef int EdgeIDType;
// typedef float TimeStampType;

template<typename T>
inline py::array vec2npy(const std::vector<T> &vec)
{
    // need to let python garbage collector handle C++ vector memory 
    // see https://github.com/pybind/pybind11/issues/1042
    // non-copy value transfer
    auto v = new std::vector<T>(vec);
    auto capsule = py::capsule(v, [](void *v)
                               { delete reinterpret_cast<std::vector<T> *>(v); });
    return py::array(v->size(), v->data(), capsule);
    // return py::array(vec.size(), vec.data());
}

class TemporalGraphBlock
{
    public:
        std::vector<NodeIDType> row;
        std::vector<NodeIDType> col;
        std::vector<NodeIDType> nodes;
        // NodeIDType dim_in, dim_out;
        // double ptr_time = 0;
        // double search_time = 0;
        // double sample_time = 0;
        // double tot_time = 0;
        // double coo_time = 0;

        TemporalGraphBlock(){}

        TemporalGraphBlock(std::vector<NodeIDType> &_row, std::vector<NodeIDType> &_col,
                           std::vector<NodeIDType> &_nodes):
                           row(_row), col(_col), nodes(_nodes){}
};

TemporalGraphBlock neighbor_sample_from_node(
        NodeIDType node, vector<NodeIDType>& row, 
        vector<NodeIDType>& col, int num_nodes, int fanout){
    int edge_num = row.size();
    int deg = 0;
    TemporalGraphBlock tgb = TemporalGraphBlock();
    srand((int)time(0));
    for(int i=0; i<edge_num; i++){
        //计算deg，循环找出邻居
        if(row[i]==node){
            deg++;
            tgb.col.push_back(col[i]);
        }
    }
    if(deg>fanout){
        //如果节点的度大于fanout 需要删除一些邻居
        for(int i=0; i<deg-fanout; i++){
            //循环随机删除邻居，共删除deg-fanout个邻居
            auto erase_iter = tgb.col.begin() + rand()%(deg-i);
            tgb.col.erase(erase_iter);
        }
    }
    tgb.row = vector<NodeIDType>(tgb.col.size(), node);
    tgb.nodes = vector<NodeIDType>(tgb.col);
    tgb.nodes.push_back(node);
    return tgb;
}

PYBIND11_MODULE(sample_cores, m)             // 定义Python模块sample_cores
{
    m.def("neighbor_sample_from_node", 
        &neighbor_sample_from_node
    );  // 定义Python函数

    py::class_<TemporalGraphBlock>(m, "TemporalGraphBlock")
        .def(py::init<std::vector<NodeIDType> &, std::vector<NodeIDType> &,
                      std::vector<NodeIDType> &>())
        .def("row", [](const TemporalGraphBlock &tgb) { return vec2npy(tgb.row); })
        .def("col", [](const TemporalGraphBlock &tgb) { return vec2npy(tgb.col); })
        .def("nodes", [](const TemporalGraphBlock &tgb) { return vec2npy(tgb.nodes); });
        // .def("tot_time", [](const TemporalGraphBlock &tgb) { return tgb.tot_time; })
        // .def("ptr_time", [](const TemporalGraphBlock &tgb) { return tgb.ptr_time; })
        // .def("search_time", [](const TemporalGraphBlock &tgb) { return tgb.search_time; })
        // .def("sample_time", [](const TemporalGraphBlock &tgb) { return tgb.sample_time; })
        // .def("coo_time", [](const TemporalGraphBlock &tgb) { return tgb.coo_time; });
}// Python模块定义结束
