import sys
import argparse
from torch.distributed.rpc import RRef, rpc_async, remote
import torch.distributed.rpc as rpc
import torch
from part.Utils import GraphData as DistGraph
from Sample.Sampler import NeighborSampler 
import time

class message_worker:
    def __init__(self,worker_size,OBSERVER_NAME):
        self.id = rpc.get_worker_info().id
        self.rank=self.id-1
        self.worker_size=worker_size
   
    #rpc demo
    def run_episode(self, agent_rref):
        state, ep_reward = self.env.reset(), 0
        for _ in range(10000):
            # send the state to the agent to get an action
            action = agent_rref.rpc_sync().select_action(self.id, state)
            # apply the action to the environment, and get the reward
            state, reward, done, _ = self.env.step(action)
            # report the reward to the agent for training purpose
            agent_rref.rpc_sync().report_reward(self.id, reward)
            # finishes after the number of self.env._max_episode_steps
            if done:
                break

    def loadGraph(self,path):
        self.graph=DistGraph(path+'/rank_{}'.format(self.rank))
        #self.graph.load_graph(path+'/rank_{}'.format(self.rank))
    
    #将所需node_list的节点拆成各分区的节点
    def split_node_part(self,node_id_list):
        part_list=[]
        for rank in range(self.worker_size):
            group_id=(self.graph.partptr[rank]<=node_id_list) & (node_id_list<self.graph.partptr[rank+1])
            part_list.append(torch.masked_select(node_id_list,group_id))
        
        return part_list
    
    def get_random_root(self,num_samples):
        return self.graph.get_globalId_by_partitionId(self.rank,torch.randint(high=self.graph.get_part_num(),size=(num_samples,)))

    def sampler(self,ob_rrefs,sampler,num_samples,**kwargs):
        
        self.ob_rrefs=ob_rrefs
        t1=time.time()
        root_list=self.get_random_root(num_samples)
        t2=time.time()
        #邻居节点list和edge list
        if('num_neighbors' in kwargs):
            num_neighbors=kwargs.get('num_neighbors')
            neighbors=root_list
            #print( self.graph.get_node_num(),num_neighbors)
            neighbors, edge_index = sampler.sample_from_nodes(root_list,self.graph.edge_index, self.graph.get_node_num(),num_neighbors)
            #print(neighbors)
        t3=time.time()
        part_feature_list=self.split_node_part(neighbors)
        t4=time.time()
        node_feature=self.get_attr(part_feature_list,self.ob_rrefs)
        print(t2-t1,t3-t2,t4-t3,time.time()-t4)

    def get_attr(self,node_feature,ob_rrefs):
        futs=[]
        part=0
        for i in range(self.worker_size):
            if(i==self.rank):
                continue
            futs.append(
                 rpc_async(
                        ob_rrefs[i].owner(),
                        ob_rrefs[i].rpc_sync().get_localattr,
                        args=(node_feature[i],)
                    )
            )
        
        for fut in futs:
            feature_part=fut.wait()
        local_feature=self.get_localattr(node_feature[self.rank])
    ''''
    #get remote attr
    def get_list_attr(self,node_list,ob_rrefs):
        feature=torch.zeros(node_list,self.graph.get_attr_size())
        sample_info={}
        for ob_rank in range(1,self.world_size): 
            sample_info[ob_rank]=[]
        futs=[]
        node_feature={}
        for i in range(node_list.size()):
            for nid in range(node_list[i]):
                part_id=self.graph.get_nodepart(nid)
                if(self.id!=part_id):
                    sample_info[part_id].append(nid)
                else:
                    node_feature[nid]=self.graph.get_nodeattr(nid)
        for ob_rank in range(1,self.world_size):
            if(ob_rank != self.id and sample_info[ob_rank]):
                sample_info=set(sample_info[ob_rank]).
                futs.append(
                    rpc_async(
                        ob_rrefs[ob_rank-1].owner(),
                        ob_rrefs[ob_rank-1].rpc_sync().get_localattr,
                        args=(sample_info[ob_rank])
                    )
                )
        for fut in futs:
            feature_part=fut.wait()
            for f in feature_part:
                node_feature[f[0]]=f[1:]
        return 
    '''
    def get_localattr(self,node_list):
    
        local_id=self.graph.get_localId_by_partitionId(self.rank,node_list)
        #print(self.rank,node_list,local_id)
        return self.graph.select_attr(local_id)

if __name__=="__main__":
    worker=message_worker(2,"observe")
    worker.loadGraph('./part/metis_2')
    sampler=NeighborSampler()
    worker.sampler([],sampler,5,num_neighbors=3)